import cartopy.crs as ccrs
import matplotlib as mpl
import matplotlib.pyplot as plt
import matplotlib.patheffects as pe
from   matplotlib.colors import ListedColormap
from   cartopy.mpl.ticker import LongitudeFormatter, LatitudeFormatter
import cartopy.feature as  cfeature
import numpy as np
from DDRW import subprogs_cartopy

import inspect

def draw_hori_ver_2FIG(slon,elon,slat,elat,data,V2D,colormapH,clevelsH,\
                       FigTitle_H,args,VER,FigTitle_V,colormapV,clevelsV,\
                       draw_top,draw_bottom,vert_info,\
                       windU2D='N',windV2D='N',\
                       scaleUV_H=0.0,maxUV_H=0.0,\
                       UVALONG='N',WVER='N',\
                       scaleUV_V=0.0,maxUV_V=0.0):
    ####################################################################
    # horizontal ########
    Fxsiz = 8
    Fysiz = 8
    shrink_v = 0.8
    fig = plt.figure(figsize=(Fxsiz,Fysiz))

    # configure for a single figure in the whole figure domain
    axH  = fig.add_subplot(211,projection=
                          ccrs.PlateCarree())
    axV  = fig.add_subplot(212)
    
    contourf = axH.contourf(
        data["LON"],data["LAT"],V2D,transform=ccrs.PlateCarree(),
        cmap=colormapH,extend="both",levels=clevelsH
    )
    
    plt.colorbar(contourf,ax=axH,shrink=shrink_v, pad= 0.03)
    
    
    ## wind ###
    if args.WIND is not None:
        if int(args.WIND) == 0:  # vector
            Q=axH.quiver(windU2D["LON"].values,windU2D["LAT"].values,\
                        windU2D,windV2D,transform=ccrs.PlateCarree(),\
                        units='inches',scale=scaleUV_H)
            # location of reference vector
            X1 = elon - (elon-slon) *0.10
            Y1 = elat + (elat-slat) *0.10
            axH.quiverkey(Q,X1,Y1,maxUV_H,str(maxUV_H)+"m/s",
                         labelpos='E',coordinates='data')

        elif int(args.WIND) == 1: # yabane
            Q=axH.barbs(windU2D["LON"].values,windU2D["LAT"].values,\
                       windU2D,windV2D,transform=ccrs.PlateCarree(),\
                       length=4)        
            
    # set drawing range lon.lat
    axH.set_extent([slon,elon,slat,elat],crs=ccrs.PlateCarree())

    # setting for xlabel and ylabel
    dlon = int(((elon-slon)/4.0)*10.0)/10.0
    dlat = int(((elat-slat)/4.0)*10.0)/10.0
    xticks0 = np.arange(slon,elon+0.01,dlon)
    xticks = np.round(xticks0,1)
    yticks0 = np.arange(slat,elat+0.01,dlat)
    yticks = np.round(yticks0,1)
    axH.set_xticks(xticks,crs=ccrs.PlateCarree())
    axH.set_yticks(yticks,crs=ccrs.PlateCarree())
    lon_formatter = LongitudeFormatter(zero_direction_label=True)
    lat_formatter = LatitudeFormatter()
    axH.xaxis.set_major_formatter(lon_formatter)
    axH.yaxis.set_major_formatter(lat_formatter)

    # Draw Fig Title
    axH.set_title(FigTitle_H,loc="center")
    
    # Boundary for prefecture
    countries_10m  = cfeature.NaturalEarthFeature('cultural',
                     'admin_1_states_provinces_lines',
                     '10m', edgecolor='black',facecolor='none')
    
    # draw coastline
    axH.coastlines()

    # draw boundary of prefecture
    axH.add_feature(countries_10m, linestyle=':', linewidth=0.5, alpha=0.8)

    # plot observation location
    #xobs = 141.523
    #yobs =  35.9
    #xobs = 141.703
    #yobs =  35.9
    #ax.plot(xobs,yobs,markersize=10, marker='x',color='black')

    # plot vertical cross section location
    vert_lineX = (vert_info[0],vert_info[2])
    vert_lineY = (vert_info[1],vert_info[3])
    lw= 3
    axH.plot(vert_lineX,vert_lineY,"-",color="black",linewidth=lw,\
             path_effects=[pe.Stroke(linewidth=lw+2,foreground='white'),\
                           pe.Normal()])

    
    #########################################################################
    # limit draw range                       
    if(draw_top > draw_bottom + 0.01):
        VER = VER.sel(LEV=slice(draw_bottom,draw_top))
        if args.WIND is not None:
            WVER = WVER.sel(LEV=slice(draw_bottom,draw_top))
            UVALONG = UVALONG.sel(LEV=slice(draw_bottom,draw_top))


    
    # check P-coordinate ###########
    if VER["LEV"][1] < VER["LEV"][0]:
        print("P-coordinate")
        NZ = len(VER["LEV"])
        #axV.set_yscale("log")
        # reverse vertical direction
        axV.set_ylim(VER["LEV"][0], VER["LEV"][NZ-1])
        contourf = axV.contourf(VER["DISTANCE"],VER["LEV"],VER,\
                                cmap=colormapV,levels=clevelsV,extend="both")
    else:
        
        TMP = VER.values
        XXX = VER["DISTANCE"].values
        YYY = VER["LEV"].values

        contourf = axV.contourf(XXX,YYY,TMP,\
                                cmap=colormapV,levels=clevelsV,\
                                extend="both")
        gray_cmap = ListedColormap([(0,0,0,0),'gray'])
        nan_mask = np.isnan(TMP).astype(int)
        extent=[XXX.min(),XXX.max(),YYY.min(),YYY.max()]
        axV.imshow(nan_mask,origin='lower',extent=extent,\
                   cmap=gray_cmap,aspect="auto")
        
    plt.colorbar(contourf,ax=axV,shrink=shrink_v, pad= 0.03)
    
    if args.WIND is not None:
       if int(args.WIND) == 0:  # vector  
            Q=axV.quiver(WVER["DISTANCE"].values,WVER["LEV"].values,\
                        UVALONG,WVER,\
                        units='inches',scale=scaleUV_V)
            X1 = Fxsiz *0.8
            Y1 = Fysiz *0.45
            axV.quiverkey(Q,X1,Y1,maxUV_V,str(maxUV_V)+"m/s",
                         labelpos='E',coordinates='inches')

    # setting for xlabel and ylabel
    axV.set_xlabel("distance (km)", size=14)
    if VER["LEV"][1] < VER["LEV"][0]:
        axV.set_ylabel("pressure (hPa)", size=14)
    else:
        axV.set_ylabel("altitude (m)", size=14)

    # Draw Fig Title
    axV.set_title(FigTitle_V,loc="center")

    
    if args.OUTFIG is not None:
            plt.savefig(args.OUTFIG)

    # draw figure#
    plt.show()
